<?php

class Database
{
	/**
	 * @return Database
	 */
	public static function getLink()
	{
		return new Database(_MYSQL_HOST, _MYSQL_DBAS, _MYSQL_USER, _MYSQL_PASS);
	}

	private $link;

	function __construct($host, $database, $user, $password)
	{
		$this->link = new mysqli($host, $user, $password, $database);

		if ($this->getInternalLink()->connect_errno)
		{
			die("Connect failed: ".$this->getInternalLink()->connect_error);
		}

		if (!$this->getInternalLink()->set_charset("utf8mb4")) {
			die("Error loading character set utf8mb4: " . $this->getInternalLink()->error);
		}

	}

	/**
	 * @return \mysqli
	 */
	private function getInternalLink()
	{
		return $this->link;
	}

	public function close()
	{
		if ($this->link)
		{
			$this->getInternalLink()->close();
		}
	}

	public function query($_q, bool $handleError = true)
	{
		$query = $this->getInternalLink()->query($_q);
		if (!$query)
		{
			if ($handleError)
				$this->handleError($_q, $this->getInternalLink()->error);
		}

		return $query;
	}

	public function handleError($query, $errorMessage)
	{
		mail("cp@pulseone.at", "covid19at.net import error", "<b>Error!</b><br /><br />".$errorMessage."<br /><br />".$query, "From: no_reply@pulseone.at");
		throw new Exception("Database Error!".PHP_EOL.$query.PHP_EOL.$errorMessage);
	}

	public function get($_q, $single = false)
	{

		$result = $this->getInternalLink()->query($_q);
		if ($result)
		{
			$r = [];
			if ($single)
			{
				return $result->fetch_assoc();
			}

			while ($row = $result->fetch_assoc())
			{
				$r[] = $row;
			}
			$result->free();
			return $r;
		}
		else
		{
			$this->handleError($_q, $this->getInternalLink()->error);
			return false;
		}
	}

	public function filter($_t)
	{
		return $this->getInternalLink()->real_escape_string($_t);
	}

	/**
	 * @param $_q
	 *
	 * @return int
	 */
	public function count($_q)
	{
		if ($result = $this->getInternalLink()->query($_q))
		{
			$row_cnt = $result->num_rows;
			$result->close();
			return (int)$row_cnt;
		}
	}

	/**
	 * @param string $s
	 *
	 * @return string
	 */
	public function quotedStringOrNull(string $s)
	{
		if (strlen(trim($s)) <1)
			return "NULL";
		return "\"".$s."\"";
	}

	/**
	 * @param $s
	 *
	 * @return int|string
	 */
	public function intOrNull($s)
	{
		if (!is_numeric($s))
			return "NULL";
		if (is_bool($s))
			return "NULL";
		return (int)$s;

	}
}